"""Provide abstract models for AIOB."""

from __future__ import annotations

from abc import ABC, abstractmethod
from dataclasses import dataclass, field
from typing import Any

from aiob.api import config


data_metas: tuple[str, ...] = (
    "id",
    "create_time",
    "update_time",
    "title",
    "slug",
    "author",
    "feature_image",
    "category",
    "tags",
    "extras",
)


@dataclass
class Data:
    """The Data Class, including id,source,content and other metadatas."""

    source: type[SourceBase] | None
    id: str
    content: str
    create_time: str
    update_time: str
    title: str = ""
    dests: list[type[DestinationBase]] = field(default_factory=list)
    slug: str = ""
    author: str = ""
    feature_image: str = ""
    category: str = ""
    tags: list[str] = field(default_factory=list)
    extras: dict[str, str] = field(default_factory=dict)

    def __post_init__(self) -> None:
        """Post Init function, set some default values accordingly."""
        if self.title == "":
            self.title = self.id


class SourceBase(ABC):
    """Abstract base class for SourceClass."""

    name: str

    @classmethod
    @abstractmethod
    async def get_opt_seq(cls) -> list[OptBase]:
        """Generate the operation sequence by monitoring changes in source.

        Returns:
            list[OptBase]: The list of operations since the last execution.
        """
        pass

    @classmethod
    def get_conf(cls, key: str, default: Any | None = None) -> Any:
        """Get configuration in the corresponding configu section by the name of current Source.

        Args:
          key: str:
          default: Any | None:  (Default value = None)

        Returns:
          config value.

        """
        return config.settings.get(f"Source.{cls.name}.{key}", default)


@dataclass  # type: ignore
class DestinationBase(ABC):
    """Abstract base class for DestinationClass."""

    name: str

    @classmethod
    @abstractmethod
    async def add(cls, data: Data) -> None:
        """Perform adding in the destination.

        Args:
            data (Data): The Data to add.
        """
        pass

    @classmethod
    @abstractmethod
    async def delete(cls, data: Data) -> None:
        """Perform deleting in the destination.

        Args:
            data (Data): The Data to delete.
        """
        pass

    @classmethod
    @abstractmethod
    async def change(cls, data: Data) -> None:
        """Perform changing a pre-existing data in the destination.

        Args:
            data (Data): The new data.
        """
        pass

    @classmethod
    def get_conf(cls, key: str, default: Any | None = None) -> Any:
        """Get configuration in the corresponding configu section by the name of current Source.

        Args:
          key: str:
          default: Any | None:  (Default value = None)

        Returns:
          config value.
        """
        return config.settings.get(f"Destination.{cls.name}.{key}", default)


@dataclass  # type: ignore
class OptBase(ABC):
    """Abstract base class for Opts."""

    data: Data

    @abstractmethod
    async def execute(self) -> None:
        """Execute the operation accordingly."""
        pass


class NoDestException(Exception):
    """Exception to indicate that the Data class hasn't been assigned to a Destination yet."""

    pass



"""Database utility for AIOB. Store Datas."""

from __future__ import annotations

import atexit

from typing import Any, Iterable

from aiob.api import config, plugin_loader
from aiob.api.model import Data, DestinationBase, SourceBase
from tinydb import TinyDB, where
from tinydb.middlewares import CachingMiddleware
from tinydb.queries import QueryLike
from tinydb.storages import JSONStorage


db: TinyDB


def init_db() -> None:
    """Initialize the database. Executed when loading the module, but can be manually called later to reinitialize."""
    global db
    db = TinyDB(config.settings.db_path, storage=CachingMiddleware(JSONStorage))


init_db()


def query_src_datas(src: type[SourceBase]) -> list[Data]:
    """Query for all the Datas in a specific Source.

    Args:
        src (type[SourceBase]): The source to query in.

    Returns:
        list[Data]: A list of Data whose Source equals to the given one.
    """
    global db
    return [_parse_to_data(x) for x in db.search(where("source") == src.name)]


def _eq_data(data: Data) -> QueryLike:
    return (
        where("source") == data.source.name if data.source is not None else None
    ) and where("id") == data.id


def query_data(src: type[SourceBase], id: str) -> Data | None:
    """Query for a Data.

    Args:
        src (type[SourceBase]): The Source to search in.
        id (str): The id of the Data.

    Returns:
        Data | None: None if not found.
    """
    global db
    ret = db.search(where("source") == src.name and where("id") == id)
    if len(ret) <= 0:
        return None
    return _parse_to_data(ret[0])


def _parse_value(obj: Any) -> Any:
    if isinstance(obj, type) and (
        issubclass(obj, SourceBase) or issubclass(obj, DestinationBase)
    ):
        return obj.name
    if isinstance(obj, list):
        return [_parse_value(x) for x in obj]
    return obj


def _parse_from_data(data: Data) -> dict:
    return {
        key: _parse_value(value)
        for key, value in data.__dict__.items()
        if not key.startswith("__")
    }


def _parse_to_data(dict: dict) -> Data:
    data = Data(**dict)
    data.source = plugin_loader.get_source_from_name(dict["source"])
    new_dests: list[type[DestinationBase]] = []
    for x in dict["dests"]:
        ret = plugin_loader.get_destination_from_name(x)
        if ret is not None:
            new_dests.append(ret)
    data.dests = new_dests
    return data


def add_data(data: Data) -> None:
    """Add a Data to the database.

    Args:
        data (Data): The Data to add.
    """
    global db
    db.insert(_parse_from_data(data))


def add_datas(datas: Iterable[Data]) -> None:
    """Add multiple Datas to the database at once.

    Args:
        datas (Iterable[Data]): The Datas to add.
    """
    global db
    db.insert_multiple([_parse_from_data(data) for data in datas])


def del_data(data: Data) -> None:
    """Delete a Data from the database.

    Args:
        data (Data): The Data to delete.
    """
    global db
    db.remove(_eq_data(data))


def change_data(data: Data) -> None:
    """Change a Data in the database. Old Data will be queried basing on Source and id.

    Args:
        data (Data): New data.
    """
    global db
    db.update(_parse_from_data(data), _eq_data(data))


@atexit.register
def close_db() -> None:
    """Close the database connection. Will be automatically executed when unloading the module."""
    global db
    if db is not None and db._opened:
        db.close()



"""Config Utility for AIOB."""

from __future__ import annotations

import os

from dynaconf import Dynaconf, Validator


settings = Dynaconf(
    envvar_prefix="AIOB",
    settings_files=[os.getcwd() + "/settings.toml"],
    validators=[
        Validator("db_path", default="db.json"),
    ],
)



"""File Markdown Source Plugin."""

from __future__ import annotations

import asyncio
import os
import pathlib

from datetime import datetime
from typing import Any, Coroutine

import aiofiles
import frontmatter

from aiob.api import db
from aiob.api.model import Data, OptBase, SourceBase
from aiob.api.opts import AddOpt, ChangeOpt, DelOpt
from aiob.api.plugin_loader import source_class


def _get_isotime(timestamp: float) -> str:
    return datetime.fromtimestamp(timestamp).isoformat()


async def _check_del(data: Data) -> DelOpt | None:
    if "origin_path" not in data.extras:
        return None
    path = pathlib.Path(data.extras["origin_path"])
    if path.exists():
        return None
    return DelOpt(data)


@source_class
class Markdown(SourceBase):
    """File Markdown Source Class."""

    name = "src_file_markdown"

    @classmethod
    async def get_opt_seq(cls) -> list[OptBase]:
        """Get the operation sequence.

        Returns:
            list[OptBase]: The list of Opts generated.
        """
        tasks: list[Coroutine[Any, Any, OptBase | None]] = []
        paths = cls.get_conf("paths", [])
        for root in paths:
            if not os.path.exists(root):
                continue
            for _, _, filenames in os.walk(root):
                for file in filenames:
                    mdpath = pathlib.Path(os.path.join(root, file))
                    tasks.append(cls._get_opt(mdpath))
        add_change_seq: list[OptBase] = await asyncio.gather(*tasks)

        # DelOpts
        pass
        olds: list[Data] = db.query_src_datas(cls)
        tasks = [_check_del(x) for x in olds]
        del_seq: list[OptBase] = await asyncio.gather(*tasks)
        return add_change_seq + del_seq

    @classmethod
    async def _get_opt(cls, mdpath: pathlib.Path) -> OptBase | None:
        async with aiofiles.open(mdpath, "r") as f:
            content = await f.read()
            data = cls._parse(mdpath, content)
        old: Data | None = db.query_data(cls, data.id)
        if old is None:
            return AddOpt(data)
        if data.update_time <= old.update_time:
            return None
        data.dests = old.dests
        return ChangeOpt(data)

    @classmethod
    def _parse(cls, mdpath: pathlib.Path, content: str) -> Data:
        post: frontmatter.Post = frontmatter.loads(content)
        id = mdpath.name.removesuffix(".md")
        title = id
        actual_content = post.content
        stat = mdpath.stat()
        create_time = _get_isotime(stat.st_ctime)
        update_time = _get_isotime(stat.st_mtime)
        data = Data(
            cls,
            id=id,
            title=title,
            content=actual_content,
            create_time=create_time,
            update_time=update_time,
        )
        data.extras["origin_path"] = str(mdpath)

        data.author = post.get("author", data.author)
        data.category = post.get("category", data.category)
        data.feature_image = post.get("feature_image", data.feature_image)
        data.slug = post.get("slug", data.slug)
        data.tags = post.get("tags", data.tags)
        data.extras.update(post.get("extras", {}))
        data.id = post.get("id", data.id)
        data.title = post.get("title", data.title)
        data.create_time = post.get("create_time", data.create_time)
        data.update_time = post.get("update_time", data.update_time)

        return data



"""Plugin loading utility for AIOB."""

from __future__ import annotations

import importlib
import os
import pkgutil
import sys

import aiob.api.Destinations
import aiob.api.Sources

from aiob.api.model import DestinationBase, SourceBase


src_list: list[type[SourceBase]] = []
dest_list: list[type[DestinationBase]] = []
load_path: list[str] = (
    [
        os.getcwd() + "/Sources",
        os.getcwd() + "/Destinations",
    ]
    + list(aiob.api.Sources.__path__)
    + list(aiob.api.Destinations.__path__)
)
sys.path = sys.path + load_path


def load_externals() -> None:
    """Load external modules from syspaths and Sources/Destination dirs and EntryPoints."""
    # loading from load_path
    for x in pkgutil.walk_packages(path=load_path):
        importlib.import_module(x.name)

    import sys

    if sys.version_info < (3, 10):
        from importlib_metadata import entry_points
    else:
        from importlib.metadata import entry_points

    plugins = entry_points(group="aiob.plugins")  # type: ignore
    for x in plugins:  # type: ignore
        x.load()  # type: ignore


def source_class(cls: type[SourceBase]) -> type[SourceBase]:
    """Decorate a SourceClass. Once applied, the SourceClass will be loaded into src_list for use.

    Args:
        cls (type[SourceBase]): The SourceClass to decorate.
    """
    src_list.append(cls)
    return cls


def destination_class(cls: type[DestinationBase]) -> type[DestinationBase]:
    """Decorate a Destination. Once applied, the DestinationClass will be loaded into dest_list for use.

    Args:
        cls (type[DestinationBase]): The DestinationClass to decorate.
    """
    dest_list.append(cls)
    return cls


def get_source_from_name(name: str) -> type[SourceBase] | None:
    """Query a loaded SourceClass from its name.

    Args:
        name (str): The name of the desired SourceClass.

    Returns:
        type[SourceBase] | None: None if not found.
    """
    for x in src_list:
        if x.name == name:
            return x
    return None


def get_destination_from_name(name: str) -> type[DestinationBase] | None:
    """Query a loaded DestinationClass from its name.

    Args:
        name (str): The name of the desired DestinationClass.

    Returns:
        type[DestinationBase] | None: None if not found.
    """
    for x in dest_list:
        if x.name == name:
            return x
    return None


# at the bottom of this module to avoid circular importing(for the decorators to work)
load_externals()



"""Bridge Module to perform some user-end operations in AIOB."""

from __future__ import annotations

import asyncio

from typing import Iterable

from aiob.api.model import OptBase
from aiob.api.plugin_loader import src_list


async def get_all_opt_seq() -> list[OptBase]:
    """Iterate all sourceclasses and get operation sequences.

    Returns:
        list[OptBase]: The combined Operation List.
    """
    tasks = [x.get_opt_seq() for x in src_list]
    opt_seqs = await asyncio.gather(*tasks)
    all_opt_seq = []
    for x in opt_seqs:
        all_opt_seq.extend(x)
    return all_opt_seq


async def exec_opts(opts: Iterable[OptBase]) -> None:
    """Execute the given Operations.

    Args:
        opts (Iterable[OptBase]): Operations to execute.
    """
    tasks = [x.execute() for x in opts]
    await asyncio.gather(*tasks)



"""Main module for AIOB CLI."""

from __future__ import annotations

import asyncio

import typer

from aiob.api import bridge, db
from aiob.api.model import Data, OptBase
from aiob.api.plugin_loader import dest_list


def _select_dest(data: Data) -> None:
    typer.echo(data)
    for i, x in enumerate(dest_list):
        typer.echo(f"{i}:{x.name}")
    while True:
        sel_dest = int(typer.prompt("Please select a Destination for this Data:"))
        dest = dest_list[sel_dest]
        if dest not in data.dests:
            data.dests.append(dest)
        finish = typer.confirm("Finished?", True)
        if finish:
            break


def main() -> None:
    """Execute main function for AIOB CLI."""
    typer.echo("AIOB running...")
    db.init_db()
    opts: list[OptBase] = [
        x for x in asyncio.run(bridge.get_all_opt_seq()) if x is not None
    ]
    for x in opts:
        if len(x.data.dests) <= 0:
            _select_dest(x.data)
    asyncio.run(bridge.exec_opts(opts))
    typer.echo("Finished!")



"""Operation Classes Implement."""

from __future__ import annotations

import asyncio

from aiob.api import db
from aiob.api.model import NoDestException, OptBase


class AddOpt(OptBase):
    """Add Operation. Add a new Data to the destinations."""

    async def execute(self) -> None:
        """Execute the add operation.

        Raises:
            NoDestException: Exception that no dests are assigned for the data.
        """
        if self.data.dests == []:
            raise NoDestException(self)
        tasks = [dest.add(self.data) for dest in self.data.dests]
        await asyncio.gather(*tasks)
        db.add_data(self.data)


class DelOpt(OptBase):
    """Delete Operation. Delete an existing Data from the destinations."""

    async def execute(self) -> None:
        """Execute the delete operation.

        Raises:
            NoDestException: Exception that no dests are assigned for the data.
        """
        if self.data.dests == []:
            raise NoDestException(self)
        tasks = [dest.delete(self.data) for dest in self.data.dests]
        await asyncio.gather(*tasks)
        db.del_data(self.data)


class ChangeOpt(OptBase):
    """Change Operation. Change an old Data to a new one."""

    async def execute(self) -> None:
        """Execute the change operation.

        Raises:
            NoDestException: Exception that no dests are assigned for the data.
        """
        if self.data.dests == []:
            raise NoDestException(self)
        tasks = [dest.change(self.data) for dest in self.data.dests]
        await asyncio.gather(*tasks)
        db.change_data(self.data)



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['aiob',
 'aiob.api',
 'aiob.api.Destinations.file_markdown',
 'aiob.api.Sources.src_file_markdown',
 'aiob.cli']

package_data = \
{'': ['*']}

install_requires = \
['aiofiles>=0.8.0,<0.9.0',
 'aiohttp>=3.8.1,<4.0.0',
 'colorama>=0.4.4,<0.5.0',
 'dynaconf>=3.1.7,<4.0.0',
 'python-frontmatter>=1.0.0,<2.0.0',
 'shellingham>=1.4.0,<2.0.0',
 'tinydb>=4.7.0,<5.0.0',
 'typer[all]>=0.4.1,<0.5.0']

extras_require = \
{':python_version < "3.10"': ['importlib-metadata>=4.11.3,<5.0.0']}

entry_points = \
{'console_scripts': ['aiob = aiob.cli.main:main']}

setup_kwargs = {
    'name': 'aiob',
    'version': '0.0.1',
    'description': 'All In One Bridge.',
    'long_description': '<div id="top"></div>\n\n<div align="center">\n  <a href="https://github.com/Clouder0/AIOB">\n    <img src="logo.jpg" alt="Logo">\n  </a>\n\n<h3 align="center">AIOB</h3>\n  <p align="center">\n    All In One Bridge for your Datas\n    <br />\n    <a href="https://github.com/Clouder0/AIOB"><strong>Explore the docs »</strong></a>\n  </p>\n</div>\n\n## 📜 TOC\n\n<details>\n  <summary>Table of Contents</summary>\n\n- [🌟 Badges](#🌟-badges)\n- [💡 Introduction](#💡-introduction)\n- [✨ Features](#✨-features)\n- [🎏 Getting Started](#🎏-getting-started)\n- [🗺️ Roadmap](#🗺️-roadmap)\n- [❓ Faq](#❓-faq)\n- [💌 Contributing](#💌-contributing)\n- [🙏 Acknowledgment](#🙏-acknowledgment)\n- [📖 License](#📖-license)\n- [📧 Contact](#📧-contact)\n\n</details>\n\n## 🌟 Badges\n\n[![Test][github-action-test-shield]][github-action-test-url]\n[![Codecov][codecov-shield]][codecov-url]\n[![pre-commit-ci][pre-commit-ci-shield]][pre-commit-ci-url]\n[![pepy-shield]][pepy-url]\n\n[![release-shield]][release-url]\n[![pyversions-shield]][pyversions-url]\n[![Contributors][contributors-shield]][contributors-url]\n[![Forks][forks-shield]][forks-url]\n[![Stargazers][stars-shield]][stars-url]\n[![Issues][issues-shield]][issues-url]\n[![Apache License 2.0][license-shield]][license-url]\n[![CodeFactor-shield]][CodeFactor-url]\n[![code-style-black-shield]][code-style-black-url]\n\n<!-- INTRODUCTION -->\n## 💡 Introduction\n\n**Unfortunately, AIOB is still under initial development and hasn\'t been prepared for users.**  \n**Several embeded sources/destinations will be added until the very first official release. Salient changes might be made to optimize project structures and so on.**  \n**Please wait until version 0.1.0 is published.**  \n\n<p align="right">(<a href="#top">back to top</a>)</p>\n\n<!-- FEATURES -->\n## ✨ Features\n\n<p align="right">(<a href="#top">back to top</a>)</p>\n\n<!-- GETTING STARTED -->\n## 🎏 Getting Started\n\n<details> <summary>Click Here to Get Started Instantly.</summary>\n\nTODO.\n\n### Installation\n\n<p align="right">(<a href="#top">back to top</a>)</p>\n\n### Command Line Interface\n\n<p align="right">(<a href="#top">back to top</a>)</p>\n\n### Configuration\n\n<p align="right">(<a href="#top">back to top</a>)</p>\n\n### Plugin System\n\n<p align="right">(<a href="#top">back to top</a>)</p>\n\n</details>\n\n<!-- ROADMAP -->\n## 🗺️ Roadmap\n\nPlease check out our [Github Project](https://github.com/users/Clouder0/projects/2).\n\nSee the [open issues](https://github.com/Clouder0/AIOB/issues) for a full list of proposed features (and known issues).\n\n<p align="right">(<a href="#top">back to top</a>)</p>\n\n<!-- FAQ -->\n## ❓ FAQ\n\n<p align="right">(<a href="#top">back to top</a>)</p>\n\n<!-- CONTRIBUTING -->\n## 💌 Contributing\n\nContributions are what make the open source community such an amazing place to learn, inspire, and create. Any contributions you make are **greatly appreciated**.\n\nIf you have a suggestion that would make this better, please fork the repo and create a pull request. You can also simply open an issue with the tag "enhancement".\nDon\'t forget to give the project a star! Thanks again!\n\n1. Fork the Project\n2. Create your Feature Branch (`git checkout -b feature/AmazingFeature`)\n3. Commit your Changes (`git commit -m \'Add some AmazingFeature\'`)\n4. Push to the Branch (`git push origin feature/AmazingFeature`)\n5. Open a Pull Request\n\nDon\'t forget to see our [Contributing Guideline](https://github.com/Clouder0/AIOB/blob/main/CONTRIBUTING.md) for details.\n\n<p align="right">(<a href="#top">back to top</a>)</p>\n\n## 🙏 Acknowledgment\n\nThere are various open source projects that AIOB depends on, without which this tool wouldn\'t exist. Credits to them!\n\n<details> <summary>List</summary>\n\n- [tinydb](https://github.com/msiemens/tinydb), MIT License\n- [Dynaconf](https://github.com/rochacbruno/dynaconf), MIT License\n- [aiofiles](https://github.com/Tinche/aiofiles), Apache License 2.0\n- [python-frontmatter](https://github.com/eyeseast/python-frontmatter), MIT License\n- [typer](https://github.com/tiangolo/typer), MIT License\n- [aiohttp](https://github.com/aio-libs/aiohttp), Apache License 2.0\n- [importlib_metadata](https://github.com/python/importlib_metadata), Apache License 2.0\n\n</details>\n\n<p align="right">(<a href="#top">back to top</a>)</p>\n\n## 📖 License\n\nDistributed under the Apache License 2.0. See `LICENSE` for more information.\n\n<p align="right">(<a href="#top">back to top</a>)</p>\n\n## 📧 Contact\n\nClouder0\'s email: clouder0@outlook.com\n\nProject Link: [https://github.com/Clouder0/AIOB](https://github.com/Clouder0/AIOB)\n\n<p align="right">(<a href="#top">back to top</a>)</p>\n\n<!-- MARKDOWN LINKS & IMAGES -->\n<!-- https://www.markdownguide.org/basic-syntax/#reference-style-links -->\n[contributors-shield]: https://img.shields.io/github/contributors/Clouder0/AIOB.svg?style=for-the-badge\n[contributors-url]: https://github.com/Clouder0/AIOB/graphs/contributors\n[forks-shield]: https://img.shields.io/github/forks/Clouder0/AIOB.svg?style=for-the-badge\n[forks-url]: https://github.com/Clouder0/AIOB/network/members\n[stars-shield]: https://img.shields.io/github/stars/Clouder0/AIOB.svg?style=for-the-badge\n[stars-url]: https://github.com/Clouder0/AIOB/stargazers\n[issues-shield]: https://img.shields.io/github/issues/Clouder0/AIOB.svg?style=for-the-badge\n[issues-url]: https://github.com/Clouder0/AIOB/issues\n[license-shield]: https://img.shields.io/github/license/Clouder0/AIOB.svg?style=for-the-badge\n[license-url]: https://github.com/Clouder0/AIOB/blob/main/LICENSE\n[github-action-test-shield]: https://github.com/Clouder0/AIOB/actions/workflows/test.yml/badge.svg?branch=main\n[github-action-test-url]: https://github.com/Clouder0/AIOB/actions/workflows/test.yml\n[codecov-shield]:https://codecov.io/gh/Clouder0/AIOB/branch/main/graph/badge.svg?token=D2XT099AFB\n[codecov-url]: https://codecov.io/gh/Clouder0/AIOB\n[pre-commit-ci-shield]: https://results.pre-commit.ci/badge/github/Clouder0/AIOB/main.svg\n[pre-commit-ci-url]: https://results.pre-commit.ci/latest/github/Clouder0/AIOB/main\n[code-style-black-shield]: https://img.shields.io/badge/code%20style-black-000000.svg?style=for-the-badge\n[code-style-black-url]: https://github.com/psf/black\n[pyversions-shield]: https://img.shields.io/pypi/pyversions/aiob.svg?style=for-the-badge\n[pyversions-url]: https://pypi.org/project/aiob/\n[release-shield]: https://img.shields.io/github/release/Clouder0/AIOB.svg?style=for-the-badge\n[release-url]: https://github.com/Clouder0/AIOB/releases\n[CodeFactor-shield]: https://www.codefactor.io/repository/github/clouder0/aiob/badge/main?style=for-the-badge\n[CodeFactor-url]: https://www.codefactor.io/repository/github/clouder0/aiob/overview/main\n[pepy-shield]: https://static.pepy.tech/personalized-badge/aiob?period=total&units=international_system&left_color=grey&right_color=brightgreen&left_text=Downloads\n[pepy-url]: https://pepy.tech/project/aiob\n',
    'author': 'clouder',
    'author_email': 'clouder0@outlook.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://www.github.com/Clouder0/AIOB',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



"""File Markdown Destination Plugin."""

from __future__ import annotations

import dataclasses
import os
import pathlib

import aiofiles
import frontmatter

from aiob.api.model import Data, DestinationBase, data_metas
from aiob.api.plugin_loader import destination_class


@destination_class
class Destination(DestinationBase):
    """File Markdown Destination Class."""

    name = "dest_file_markdown"

    @classmethod
    def _get_path(cls, data: Data) -> pathlib.Path:
        path = pathlib.Path(
            Destination.get_conf("path", os.getcwd() + "/") + data.title + ".md"
        )
        return path

    @classmethod
    async def add(cls, data: Data) -> None:
        """Perform Add Operation, write content to files.

        Args:
            data (Data): The Data to add.
        """
        path = cls._get_path(data)
        path.parent.mkdir(parents=True, exist_ok=True)
        async with aiofiles.open(path, "w+") as f:
            await f.write(cls._parse(data))

    @classmethod
    async def delete(cls, data: Data) -> None:
        """Perform Delete Operation, delete the corresponding file on the filesystem.

        Args:
            data (Data): The Data to delete.
        """
        path = cls._get_path(data)
        os.remove(path)

    @classmethod
    async def change(cls, data: Data) -> None:
        """Perform Change Operation, rewrite content to the corresponding file.

        Args:
            data (Data): The new Data.
        """
        path = cls._get_path(data)
        path.parent.mkdir(parents=True, exist_ok=True)
        async with aiofiles.open(path, "w+") as f:
            await f.write(cls._parse(data))

    @classmethod
    def _parse(cls, data: Data) -> str:
        if not cls.get_conf("frontmatter", True):
            return data.content

        post = frontmatter.Post(data.content)

        for x in dataclasses.fields(data):
            if x.name in data_metas and getattr(data, x.name) != x.default:
                post.metadata[x.name] = getattr(data, x.name)

        del post.metadata["title"]
        if post.metadata["tags"] == []:
            del post.metadata["tags"]
        if len(post.metadata["extras"].keys()) == 0:
            del post.metadata["extras"]

        if len(post.metadata.keys()) > 0:
            return frontmatter.dumps(post)
        else:
            return data.content
